"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkEventsContinuity = exports.emptyLoadEventsResult = exports.cursorToThreadArray = exports.threadArrayToCursor = exports.initThreadArray = void 0;

var _assert = _interopRequireDefault(require("assert"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const checkThreadArrayLength = threadArray => {
  _assert.default.strictEqual(threadArray.length, _constants.THREAD_COUNT, 'Cursor must be represented by array of 256 numbers');
};

const initThreadArray = () => {
  const threadCounters = new Array(_constants.THREAD_COUNT);
  threadCounters.fill(0);
  return threadCounters;
};

exports.initThreadArray = initThreadArray;

const threadArrayToCursor = threadArray => {
  checkThreadArrayLength(threadArray);
  const cursorBuffer = Buffer.alloc(_constants.CURSOR_BUFFER_SIZE);

  for (let i = 0; i < threadArray.length; ++i) {
    cursorBuffer.writeUIntBE(threadArray[i], i * _constants.THREAD_COUNTER_BYTE_LENGTH, _constants.THREAD_COUNTER_BYTE_LENGTH);
  }

  return cursorBuffer.toString('base64');
};

exports.threadArrayToCursor = threadArrayToCursor;

const cursorToThreadArray = cursor => {
  if (cursor == null) return initThreadArray();
  const cursorBuffer = Buffer.from(cursor, 'base64');

  _assert.default.strictEqual(cursorBuffer.length, _constants.CURSOR_BUFFER_SIZE, 'Wrong size of cursor buffer');

  const threadCounters = new Array(_constants.THREAD_COUNT);

  for (let i = 0; i < cursorBuffer.length / _constants.THREAD_COUNTER_BYTE_LENGTH; i++) {
    threadCounters[i] = cursorBuffer.readUIntBE(i * _constants.THREAD_COUNTER_BYTE_LENGTH, _constants.THREAD_COUNTER_BYTE_LENGTH);
  }

  return threadCounters;
};

exports.cursorToThreadArray = cursorToThreadArray;

const emptyLoadEventsResult = cursor => {
  return {
    cursor: cursor == null ? threadArrayToCursor(initThreadArray()) : cursor,
    events: []
  };
};

exports.emptyLoadEventsResult = emptyLoadEventsResult;

const calculateMaxThreadArray = threadArrays => {
  const maxThreadArray = initThreadArray();

  for (const threadArray of threadArrays) {
    checkThreadArrayLength(threadArray);

    for (let i = 0; i < _constants.THREAD_COUNT; ++i) {
      maxThreadArray[i] = Math.max(maxThreadArray[i], threadArray[i]);
    }
  }

  return maxThreadArray;
};

const checkEventsContinuity = (startingCursor, eventCursorPairs) => {
  const startingThreadArray = cursorToThreadArray(startingCursor);
  const tuples = eventCursorPairs.map(({
    event,
    cursor
  }) => {
    return {
      event,
      cursor,
      threadArray: cursorToThreadArray(cursor)
    };
  });

  for (let i = 0; i < tuples.length; ++i) {
    _assert.default.strictEqual(tuples[i].event.threadCounter, tuples[i].threadArray[tuples[i].event.threadId] - 1);

    if (startingThreadArray[tuples[i].event.threadId] > tuples[i].event.threadCounter) {
      return false;
    }
  }

  const maxThreadArray = calculateMaxThreadArray(tuples.map(t => t.threadArray));

  for (const t of tuples) {
    startingThreadArray[t.event.threadId]++;
  }

  for (let i = 0; i < _constants.THREAD_COUNT; ++i) {
    if (maxThreadArray[i] !== startingThreadArray[i]) {
      return false;
    }
  }

  return true;
};

exports.checkEventsContinuity = checkEventsContinuity;
//# sourceMappingURL=cursor-operations.js.map