"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.maybeThrowResourceError = exports.ResourceNotExistError = exports.ResourceAlreadyExistError = void 0;

var _errors = require("./errors");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function DefineResourceError(name, httpStatus) {
  return class extends Error {
    constructor(msg) {
      super(msg);

      _defineProperty(this, "code", void 0);

      this.name = name;
      this.code = httpStatus;
      Object.setPrototypeOf(this, new.target.prototype);
    }

    static is(err) {
      return (0, _errors.isSpecificError)(err, name);
    }

  };
}

const ResourceAlreadyExistError = DefineResourceError('ResourceAlreadyExistError', 406);
exports.ResourceAlreadyExistError = ResourceAlreadyExistError;
const ResourceNotExistError = DefineResourceError('ResourceNotExistError', 410);
exports.ResourceNotExistError = ResourceNotExistError;
const allowedResourceErrorsConstructors = Array.from([ResourceAlreadyExistError, ResourceNotExistError]);

const maybeThrowResourceError = errors => {
  if (!Array.isArray(errors) || !errors.reduce((acc, error) => acc && error instanceof Error, true)) {
    throw new Error(`Resource errors array is invalid ${errors}`);
  }

  if (errors.length === 0) {
    return;
  }

  const messages = errors.map(error => error.message).join('\n');
  const stacks = errors.map(error => error.stack).join('\n');
  const errorConstructors = errors.reduce((acc, error) => acc.add(error.constructor), new Set());
  let ErrorConstructor = errorConstructors.size === 1 ? [...errorConstructors.values()][0] : Error;

  if (!allowedResourceErrorsConstructors.includes(ErrorConstructor)) {
    ErrorConstructor = Error;
  }

  const error = new ErrorConstructor(messages);
  error.stack = stacks;
  throw error;
};

exports.maybeThrowResourceError = maybeThrowResourceError;
//# sourceMappingURL=resource-errors.js.map